@@ -11,7 +11,7 @@
 
 #include <kconfig.h>
 #include <KXMessages>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QTemporaryFile>
 #include <QFile>
 #include <QFileInfo>
@@ -21,7 +21,10 @@
 #ifndef KCMRULES
 #include "x11client.h"
 #include "client_machine.h"
+#include "main.h"
+#include "platform.h"
 #include "screens.h"
+#include "virtualdesktops.h"
 #include "workspace.h"
 #endif
 
@@ -47,7 +50,7 @@ Rules::Rules()
     , opacityactiverule(UnusedForceRule)
     , opacityinactiverule(UnusedForceRule)
     , ignoregeometryrule(UnusedSetRule)
-    , desktoprule(UnusedSetRule)
+    , desktopsrule(UnusedSetRule)
     , screenrule(UnusedSetRule)
     , activityrule(UnusedSetRule)
     , typerule(UnusedForceRule)
@@ -139,7 +142,7 @@ void Rules::readFromSettings(const RuleSettings *settings)
     READ_FORCE_RULE(opacityactive,);
     READ_FORCE_RULE(opacityinactive,);
     READ_SET_RULE(ignoregeometry);
-    READ_SET_RULE(desktop);
+    READ_SET_RULE(desktops);
     READ_SET_RULE(screen);
     READ_SET_RULE(activity);
     READ_FORCE_RULE(type, static_cast<NET::WindowType>);
@@ -219,7 +222,7 @@ void Rules::write(RuleSettings *settings) const
     WRITE_FORCE_RULE(opacityactive, Opacityactive,);
     WRITE_FORCE_RULE(opacityinactive, Opacityinactive,);
     WRITE_SET_RULE(ignoregeometry, Ignoregeometry,);
-    WRITE_SET_RULE(desktop, Desktop,);
+    WRITE_SET_RULE(desktops, Desktops,);
     WRITE_SET_RULE(screen, Screen,);
     WRITE_SET_RULE(activity, Activity,);
     WRITE_FORCE_RULE(type, Type,);
@@ -271,7 +274,7 @@ bool Rules::isEmpty() const
            && opacityactiverule == UnusedForceRule
            && opacityinactiverule == UnusedForceRule
            && ignoregeometryrule == UnusedSetRule
-           && desktoprule == UnusedSetRule
+           && desktopsrule == UnusedSetRule
            && screenrule == UnusedSetRule
            && activityrule == UnusedSetRule
            && typerule == UnusedForceRule
@@ -335,7 +338,7 @@ bool Rules::matchWMClass(const QByteArray& match_class, const QByteArray& match_
         // TODO optimize?
         QByteArray cwmclass = wmclasscomplete
                               ? match_name + ' ' + match_class : match_class;
-        if (wmclassmatch == RegExpMatch && QRegExp(QString::fromUtf8(wmclass)).indexIn(QString::fromUtf8(cwmclass)) == -1)
+        if (wmclassmatch == RegExpMatch && !QRegularExpression(QString::fromUtf8(wmclass)).match(QString::fromUtf8(cwmclass)).hasMatch())
             return false;
         if (wmclassmatch == ExactMatch && wmclass != cwmclass)
             return false;
@@ -348,7 +351,7 @@ bool Rules::matchWMClass(const QByteArray& match_class, const QByteArray& match_
 bool Rules::matchRole(const QByteArray& match_role) const
 {
     if (windowrolematch != UnimportantMatch) {
-        if (windowrolematch == RegExpMatch && QRegExp(QString::fromUtf8(windowrole)).indexIn(QString::fromUtf8(match_role)) == -1)
+        if (windowrolematch == RegExpMatch && !QRegularExpression(QString::fromUtf8(windowrole)).match(QString::fromUtf8(match_role)).hasMatch())
             return false;
         if (windowrolematch == ExactMatch && windowrole != match_role)
             return false;
@@ -361,7 +364,7 @@ bool Rules::matchRole(const QByteArray& match_role) const
 bool Rules::matchTitle(const QString& match_title) const
 {
     if (titlematch != UnimportantMatch) {
-        if (titlematch == RegExpMatch && QRegExp(title).indexIn(match_title) == -1)
+        if (titlematch == RegExpMatch && !QRegularExpression(title).match(match_title).hasMatch())
             return false;
         if (titlematch == ExactMatch && title != match_title)
             return false;
@@ -379,7 +382,7 @@ bool Rules::matchClientMachine(const QByteArray& match_machine, bool local) cons
                 && matchClientMachine("localhost", true))
             return true;
         if (clientmachinematch == RegExpMatch
-                && QRegExp(QString::fromUtf8(clientmachine)).indexIn(QString::fromUtf8(match_machine)) == -1)
+                && !QRegularExpression(QString::fromUtf8(clientmachine)).match(QString::fromUtf8(match_machine)).hasMatch())
             return false;
         if (clientmachinematch == ExactMatch
                 && clientmachine != match_machine)
@@ -442,9 +445,9 @@ bool Rules::update(AbstractClient* c, int selection)
             size = new_size;
         }
     }
-    if NOW_REMEMBER(Desktop, desktop) {
-        updated = updated || desktop != c->desktop();
-        desktop = c->desktop();
+    if NOW_REMEMBER(Desktops, desktops) {
+        updated = updated || desktops != c->desktopIds();
+        desktops = c->desktopIds();
     }
     if NOW_REMEMBER(Screen, screen) {
         updated = updated || screen != c->screen();
@@ -561,11 +564,23 @@ APPLY_FORCE_RULE(opacityactive, OpacityActive, int)
 APPLY_FORCE_RULE(opacityinactive, OpacityInactive, int)
 APPLY_RULE(ignoregeometry, IgnoreGeometry, bool)
 
-APPLY_RULE(desktop, Desktop, int)
 APPLY_RULE(screen, Screen, int)
 APPLY_RULE(activity, Activity, QStringList)
 APPLY_FORCE_RULE(type, Type, NET::WindowType)
 
+bool Rules::applyDesktops(QVector<VirtualDesktop *> &vds, bool init) const
+{
+    if (checkSetRule(desktopsrule, init)) {
+        vds.clear();
+        for (auto id : desktops) {
+            if (auto vd = VirtualDesktopManager::self()->desktopForId(id)) {
+                vds << vd;
+            }
+        }
+    }
+    return checkSetStop(desktopsrule);
+}
+
 bool Rules::applyMaximizeHoriz(MaximizeMode& mode, bool init) const
 {
     if (checkSetRule(maximizehorizrule, init))
@@ -660,7 +675,7 @@ bool Rules::discardUsed(bool withdrawn)
     DISCARD_USED_FORCE_RULE(opacityactive);
     DISCARD_USED_FORCE_RULE(opacityinactive);
     DISCARD_USED_SET_RULE(ignoregeometry);
-    DISCARD_USED_SET_RULE(desktop);
+    DISCARD_USED_SET_RULE(desktops);
     DISCARD_USED_SET_RULE(screen);
     DISCARD_USED_SET_RULE(activity);
     DISCARD_USED_FORCE_RULE(type);
@@ -776,7 +791,7 @@ CHECK_FORCE_RULE(OpacityActive, int)
 CHECK_FORCE_RULE(OpacityInactive, int)
 CHECK_RULE(IgnoreGeometry, bool)
 
-CHECK_RULE(Desktop, int)
+CHECK_RULE(Desktops, QVector<VirtualDesktop *>)
 CHECK_RULE(Activity, QStringList)
 CHECK_FORCE_RULE(Type, NET::WindowType)
 CHECK_RULE(MaximizeVert, MaximizeMode)
@@ -789,18 +804,19 @@ MaximizeMode WindowRules::checkMaximize(MaximizeMode mode, bool init) const
     return static_cast< MaximizeMode >((vert ? MaximizeVertical : 0) | (horiz ? MaximizeHorizontal : 0));
 }
 
-int WindowRules::checkScreen(int screen, bool init) const
+AbstractOutput *WindowRules::checkOutput(AbstractOutput *output, bool init) const
 {
-    if ( rules.count() == 0 )
-        return screen;
-    int ret = screen;
-    for ( QVector< Rules* >::ConstIterator it = rules.constBegin(); it != rules.constEnd(); ++it ) {
-        if ( (*it)->applyScreen( ret, init ))
-            break;
+    if (rules.isEmpty()) {
+        return output;
     }
-    if (ret >= Screens::self()->count())
-        ret = screen;
-    return ret;
+    int ret = kwinApp()->platform()->enabledOutputs().indexOf(output);
+    for (Rules *rule : rules) {
+        if (rule->applyScreen(ret, init)) {
+            break;
+        }
+    }
+    AbstractOutput *ruleOutput = kwinApp()->platform()->findOutput(ret);
+    return ruleOutput ? ruleOutput : output;
 }
 
 CHECK_RULE(Minimize, bool)
@@ -849,11 +865,11 @@ void AbstractClient::applyWindowRules()
     QRect orig_geom = QRect(pos(), adjustedSize());   // handle shading
     QRect geom = client_rules->checkGeometry(orig_geom);
     if (geom != orig_geom)
-        setFrameGeometry(geom);
+        moveResize(geom);
     // MinSize, MaxSize handled by Geometry
     // IgnoreGeometry
-    setDesktop(desktop());
-    workspace()->sendClientToScreen(this, screen());
+    setDesktops(desktops());
+    workspace()->sendClientToOutput(this, output());
     setOnActivities(activities());
     // Type
     maximize(maximizeMode());
@@ -1013,7 +1029,9 @@ void RuleBook::load()
     } else {
         m_config->reparseConfiguration();
     }
-    m_rules = RuleBookSettings(m_config).rules().toList();
+    RuleBookSettings book(m_config);
+    book.load();
+    m_rules = book.rules().toList();
 }
 
 void RuleBook::save()
@@ -1099,7 +1117,7 @@ void RuleBook::setUpdatesDisabled(bool disable)
 {
     m_updatesDisabled = disable;
     if (!disable) {
-        foreach (X11Client *c, Workspace::self()->clientList())
+        Q_FOREACH (X11Client *c, Workspace::self()->clientList())
             c->updateWindowRules(Rules::All);
     }
 }
