@@ -15,12 +15,16 @@
 #include "scriptingutils.h"
 #include "workspace_wrapper.h"
 #include "screenedgeitem.h"
-#include "scripting_model.h"
 #include "scripting_logging.h"
+#include "thumbnailitem.h"
 
+#include "v2/clientmodel.h"
+#include "v3/clientmodel.h"
+
+#include "input.h"
 #include "options.h"
 #include "screenedge.h"
-#include "thumbnailitem.h"
+#include "virtualdesktops.h"
 #include "workspace.h"
 #include "x11client.h"
 // KDE
@@ -430,8 +434,8 @@ QList<QAction *> KWin::Script::actionsForUserActionMenu(KWin::AbstractClient *cl
     QList<QAction *> actions;
     actions.reserve(m_userActionsMenuCallbacks.count());
 
-    for (QJSValue callback : m_userActionsMenuCallbacks) {
-        QJSValue result = callback.call({ m_engine->toScriptValue(client) });
+    for (QJSValue callback : qAsConst(m_userActionsMenuCallbacks)) {
+        const QJSValue result = callback.call({ m_engine->toScriptValue(client) });
         if (result.isError()) {
             continue;
         }
@@ -575,9 +579,10 @@ QVariant KWin::JSEngineGlobalMethodsWrapper::readConfig(const QString &key, QVar
 
 void KWin::JSEngineGlobalMethodsWrapper::registerWindow(QQuickWindow *window)
 {
-    connect(window, &QWindow::visibilityChanged, this, [window](QWindow::Visibility visibility) {
-        if (visibility == QWindow::Hidden) {
-            window->destroy();
+    QPointer<QQuickWindow> guard = window;
+    connect(window, &QWindow::visibilityChanged, this, [guard](QWindow::Visibility visibility) {
+        if (guard && visibility == QWindow::Hidden) {
+            guard->destroy();
         }
     }, Qt::QueuedConnection);
 }
@@ -632,13 +637,27 @@ void KWin::Scripting::init()
     qmlRegisterType<WindowThumbnailItem>("org.kde.kwin", 2, 0, "ThumbnailItem");
     qmlRegisterType<DBusCall>("org.kde.kwin", 2, 0, "DBusCall");
     qmlRegisterType<ScreenEdgeItem>("org.kde.kwin", 2, 0, "ScreenEdgeItem");
-    qmlRegisterType<KWin::ScriptingClientModel::ClientModel>();
-    qmlRegisterType<KWin::ScriptingClientModel::SimpleClientModel>("org.kde.kwin", 2, 0, "ClientModel");
-    qmlRegisterType<KWin::ScriptingClientModel::ClientModelByScreen>("org.kde.kwin", 2, 0, "ClientModelByScreen");
-    qmlRegisterType<KWin::ScriptingClientModel::ClientModelByScreenAndDesktop>("org.kde.kwin", 2, 0, "ClientModelByScreenAndDesktop");
-    qmlRegisterType<KWin::ScriptingClientModel::ClientModelByScreenAndActivity>("org.kde.kwin", 2, 1, "ClientModelByScreenAndActivity");
-    qmlRegisterType<KWin::ScriptingClientModel::ClientFilterModel>("org.kde.kwin", 2, 0, "ClientFilterModel");
+    qmlRegisterType<ScriptingModels::V2::ClientModel>();
+    qmlRegisterType<ScriptingModels::V2::SimpleClientModel>("org.kde.kwin", 2, 0, "ClientModel");
+    qmlRegisterType<ScriptingModels::V2::ClientModelByScreen>("org.kde.kwin", 2, 0, "ClientModelByScreen");
+    qmlRegisterType<ScriptingModels::V2::ClientModelByScreenAndDesktop>("org.kde.kwin", 2, 0, "ClientModelByScreenAndDesktop");
+    qmlRegisterType<ScriptingModels::V2::ClientModelByScreenAndActivity>("org.kde.kwin", 2, 1, "ClientModelByScreenAndActivity");
+    qmlRegisterType<ScriptingModels::V2::ClientFilterModel>("org.kde.kwin", 2, 0, "ClientFilterModel");
+
+    qmlRegisterType<WindowThumbnailItem>("org.kde.kwin", 3, 0, "WindowThumbnailItem");
+    qmlRegisterType<DBusCall>("org.kde.kwin", 3, 0, "DBusCall");
+    qmlRegisterType<ScreenEdgeItem>("org.kde.kwin", 3, 0, "ScreenEdgeItem");
+    qmlRegisterType<ScriptingModels::V3::ClientModel>("org.kde.kwin", 3, 0, "ClientModel");
+    qmlRegisterType<ScriptingModels::V3::ClientFilterModel>("org.kde.kwin", 3, 0, "ClientFilterModel");
+
+    qmlRegisterSingletonType<QtScriptWorkspaceWrapper>("org.kde.kwin", 3, 0, "Workspace", [](QQmlEngine *qmlEngine, QJSEngine *jsEngine) {
+        Q_UNUSED(qmlEngine)
+        Q_UNUSED(jsEngine)
+        return new QtScriptWorkspaceWrapper();
+    });
+
     qmlRegisterType<KWin::AbstractClient>();
+    qmlRegisterType<KWin::VirtualDesktop>();
     qmlRegisterType<KWin::X11Client>();
     qmlRegisterType<QAbstractItemModel>();
 
@@ -749,7 +768,7 @@ bool KWin::Scripting::isScriptLoaded(const QString &pluginName) const
 KWin::AbstractScript *KWin::Scripting::findScript(const QString &pluginName) const
 {
     QMutexLocker locker(m_scriptsLock.data());
-    foreach (AbstractScript *script, scripts) {
+    Q_FOREACH (AbstractScript *script, scripts) {
         if (script->pluginName() == pluginName) {
             return script;
         }
@@ -760,7 +779,7 @@ KWin::AbstractScript *KWin::Scripting::findScript(const QString &pluginName) con
 bool KWin::Scripting::unloadScript(const QString &pluginName)
 {
     QMutexLocker locker(m_scriptsLock.data());
-    foreach (AbstractScript *script, scripts) {
+    Q_FOREACH (AbstractScript *script, scripts) {
         if (script->pluginName() == pluginName) {
             script->deleteLater();
             return true;
@@ -818,7 +837,7 @@ KWin::Scripting::~Scripting()
 QList< QAction * > KWin::Scripting::actionsForUserActionMenu(KWin::AbstractClient *c, QMenu *parent)
 {
     QList<QAction*> actions;
-    for (AbstractScript *s : scripts) {
+    for (AbstractScript *s : qAsConst(scripts)) {
         // TODO: Allow declarative scripts to add their own user actions.
         if (Script *script = qobject_cast<Script *>(s)) {
             actions << script->actionsForUserActionMenu(c, parent);
